// Add custom models in the map editor to a map //

void() place_model =
{
	self.touch = armor_touch;
	precache_model (self.model);
	setmodel (self, self.model);
	self.skin = 0;
	setsize (self, '-16 -16 0', '16 16 56');
};

/*QUAKED wallsprite (0 1 0) (-8 -8 -8) (8 8 8)
Places a sprite on a wall.  Angles should be opposite of face.

"model" sprite to place on wall.  Default is "progs/s_blood1.spr".
*/
void() wallsprite =
{
	if ( !self.model )
	      self.model = "progs/s_blood1.spr";

	precache_model( self.model );
	setmodel (self, self.model );

	// QuakeEd doesn't save up and down angles properly.
	if (self.angles == '0 -1 0')
		self.angles = '-90 0 0';
	else if (self.angles == '0 -2 0')
		self.angles = '90 0 0';

	// Pull the sprite away from the wall slightly to
	// get rid of z sort errors.
	makevectors(self.angles);
	setorigin( self, self.origin - ( v_forward * 0.2 ) );
	makestatic (self);
};

void() trigger_command_use =
{
	if (other.classname != "player")
		return;

	if (!other.ishuman)
		return;

	stuffcmd (other, self.message);
};

void() trigger_command =
{
	InitTrigger();
	self.touch = trigger_command_use;
	self.think = SUB_Null;
};

void() info_command_use =
{
	if (self.message)
		localcmd(self.message);
};

void() info_command =
{
	self.use = info_command_use;
	self.think = SUB_Null;
};

void() palm_tree_closed =
{
	precache_model ("progs/m_palmc.mdl");
	setmodel (self, "progs/m_palmc.mdl");
	makestatic (self);
};

void() small_bush =
{
	precache_model ("progs/bush.mdl");
	setmodel (self, "progs/bush.mdl");
	makestatic (self);
};

void() big_bush =
{
	precache_model ("progs/b_bush.mdl");
	setmodel (self, "progs/b_bush.mdl");
	makestatic (self);
};

void() wooden_crate_damage =
{
	local entity    explode;

	explode = spawn();

	explode.owner = self;
	explode.classname = "spike";

	explode.movetype = MOVETYPE_NOCLIP;
	explode.origin = self.origin + crandom()*v_forward*10 + v_up * 10 + crandom()*v_right*10;
	explode.velocity = '0 0 1';
	explode.velocity = self.velocity + crandom()*v_forward*10 + v_up * 50 + crandom()*v_right*10;
	explode.touch = SUB_Null;
	setmodel (explode, "progs/s_explod.spr");
	explode.solid = SOLID_NOT;

	explode.think =	s_explode1;
	explode.nextthink = time + 0.05;

	sound (self, CHAN_VOICE, "weapons/r_exp3.wav", 1, ATTN_NORM);

	T_RadiusDamage (self, self.owner, 40, other);

	WriteByte (MSG_BROADCAST, SVC_TEMPENTITY);
	WriteByte (MSG_BROADCAST, TE_EXPLOSION);
	WriteCoord (MSG_BROADCAST, self.origin_x);
	WriteCoord (MSG_BROADCAST, self.origin_y);
	WriteCoord (MSG_BROADCAST, self.origin_z);
	WriteCoord (MSG_BROADCAST, 1.0); // Red explosion colour
	WriteCoord (MSG_BROADCAST, 0.5); // Green explosion colour
	WriteCoord (MSG_BROADCAST, 0.0); // Blue explosion colour

	self.solid = SOLID_NOT;
	setmodel (self, "progs/d_crate.mdl");
};

void() wooden_crate_explode =
{
	self.takedamage = DAMAGE_NO;
	self.classname = "explo_box";
	self.nextthink = time + 0.1;
	self.think = wooden_crate_damage;
};

void() wooden_crate_touch =
{
	if(!(other.flags & FL_CLIENT))
		return;

	if (other.origin_z + other.mins_z + 8 >= self.absmax_z) {
		if ((!other.flags & FL_ONGROUND))
			other.flags = other.flags + FL_ONGROUND;
		return;
	}
}

void() wooden_crate =
{
	local float	oldz;

	self.solid = SOLID_BBOX;
	self.movetype = MOVETYPE_STEP;

	precache_sound ("weapons/r_exp3.wav");
	precache_model ("progs/d_crate.mdl");

	precache_model ("progs/crate.mdl");
	setmodel (self, "progs/crate.mdl");

	self.health = 25;
	self.th_die = wooden_crate_explode;
	self.touch = wooden_crate_touch;

	self.takedamage = DAMAGE_AIM;
	setsize (self, '-20 -20 0', '20 20 40');

	self.origin_z = self.origin_z + 2;
	oldz = self.origin_z;
	droptofloor();
};

//##########################################
//#### Universal ambience function      ####
//##########################################
void() ambient_general =
{
	precache_sound (self.noise);
	if(!self.speed)
		self.speed = ATTN_STATIC;
	if(!self.volume)
		self.volume = 0.5;
	ambientsound (self.origin, self.noise, self.volume, ATTN_NONE);
};

//##########################################
//#### Multipurpose sound playing entity ###
//##########################################
void() bgm_loop =
{
	if(self.ammo_shells == 0)
	{
		self.ammo_cells = FALSE;
		sound (self, self.impulse, self.noise3, self.volume, self.speed);
		if(self.target)
		{
			self.think = SUB_UseTargets;
			self.nextthink = time + self.button0;
		}
		else
			self.nextthink = 0;

		return;
	}
	else if(self.ammo_shells > 0)
		self.ammo_shells = self.ammo_shells - 1;

	sound (self, CHAN_VOICE, self.noise2, self.volume, self.speed);
	self.nextthink = time + self.button2;
};

void() bgm_use =
{
	if(self.ammo_cells)
	{
		self.ammo_cells = FALSE;
		if(self.noise3 == "none")
			self.think = SUB_Null;
		else if(self.noise3 != "misc/null.wav")
		{
			sound (self, self.impulse, self.noise3, self.volume, self.speed);
			if(self.target)
			{
				self.think = SUB_UseTargets;
				self.nextthink = time + self.button0;
			}
			else
				self.nextthink = 0;
		}
		else
		{
			sound (self, self.impulse, "misc/null.wav", self.volume, self.speed);
			if(self.target)
				SUB_UseTargets();
			self.nextthink = 0;
		}
	}
	else
	{
		self.ammo_cells = TRUE;
		if(self.noise1 != "misc/null.wav")
		{
			sound (self, self.impulse, self.noise1, self.volume, self.speed);
			self.nextthink = time + self.button1;
		}
		else
		{
			sound (self, self.impulse, self.noise2, self.volume, self.speed);
			self.nextthink = time + self.button2;
		}
		if(self.button2 > 0)
			self.think = bgm_loop;
		else
			self.think = SUB_Null;
		if(self.waitmax)
			self.ammo_shells = self.waitmax;
		else
			self.ammo_shells = -1;
	}
};

void() ambient_bgm =
{
	if(self.noise1)
		precache_sound (self.noise1);
	else
		self.noise1 = "misc/null.wav";

	if(self.noise2)
		precache_sound (self.noise2);
	else
		objerror ("no looping wav file.\n");

	if(self.noise3)
		precache_sound (self.noise3);
	else
		self.noise3 = "misc/null.wav";

	precache_sound ("misc/null.wav");

	if(!self.impulse)
		self.impulse = CHAN_VOICE;
	else if(self.impulse == -1)
		self.impulse = CHAN_AUTO;

	self.ammo_cells = FALSE;

	if(self.waitmax)
		self.ammo_shells = self.waitmax;
	else
		self.ammo_shells = -1;

	if(!self.speed)
		self.speed = ATTN_NONE;

	if(!self.volume)
		self.volume = 1;

	if(!self.targetname || (self.spawnflags & BGM_START_ON))
	{
		self.think = bgm_use;
		self.nextthink = time + 1.5;
	}

	self.use = bgm_use;
};
