void() W_SetCurrentAmmo;
void() W_SetAmmoOnly;
void() W_WeaponChange;

/* ALL LIGHTS SHOULD BE 0 1 0 IN COLOR ALL OTHER ITEMS SHOULD
BE .8 .3 .4 IN COLOR */

void() SUB_regen =
{
	self.model = self.mdl;		// restore original model
	self.solid = SOLID_TRIGGER;	// allow it to be touched again
	sound (self, CHAN_VOICE, "items/itembk2.wav", 1, ATTN_NORM);	// play respawn sound
	setorigin (self, self.origin);
};

/*QUAKED noclass (0 0 0) (-8 -8 -8) (8 8 8)
prints a warning message when spawned
*/
void() noclass =
{
	dprint ("noclass spawned at");
	dprint (vtos(self.origin));
	dprint ("\n");
	remove (self);
};



/*
============
PlaceItem

plants the object on the floor
============
*/
void() PlaceItem =
{
	local float	oldz;

	self.mdl = self.model;		// so it can be restored on respawn
	self.flags = FL_ITEM;		// make extra wide
	self.solid = SOLID_TRIGGER;
	self.movetype = MOVETYPE_TOSS;	
	self.velocity = '0 0 0';
	self.origin_z = self.origin_z + 6;
	oldz = self.origin_z;
	if (!droptofloor())
	{
		dprint ("Bonus item fell out of level at ");
		dprint (vtos(self.origin));
		dprint ("\n");
		remove(self);
		return;
	}
};

/*
============
StartItem

Sets the clipping size and plants the object on the floor
============
*/
void() StartItem =
{
	self.nextthink = time + 0.2;	// items start after other solids
	self.think = PlaceItem;
};

/*
=========================================================================

HEALTH BOX

=========================================================================
*/
//
// T_Heal: add health to an entity, limiting health to max_health
// "ignore" will ignore max_health limit
//
float (entity e, float healamount, float ignore) T_Heal =
{
	if (e.health <= 0)
		return 0;
	if ((!ignore) && (e.health >= other.max_health))
		return 0;
	healamount = ceil(healamount);

	e.health = e.health + healamount;
	if ((!ignore) && (e.health >= other.max_health))
		e.health = other.max_health;
		
	if (e.health > 250)
		e.health = 250;
	return 1;
};

/*QUAKED item_health (.3 .3 1) (0 0 0) (32 32 32) rotten megahealth
Health box. Normally gives 25 points.
Rotten box heals 5-10 points,
megahealth will add 100 health, then 
rot you down to your maximum health limit, 
one point per second.
*/

float	H_ROTTEN = 1;
float	H_MEGA = 2;
.float	healamount, healtype;
void() health_touch;
void() item_megahealth_rot;

void() item_health =
{
	if (deathmatch & DM_NO_HEALTH)
		return;

	self.touch = health_touch;

	if (self.spawnflags & H_ROTTEN)
	{
		precache_model("maps/b_bh10.bsp");

		precache_sound("items/r_item1.wav");
		setmodel(self, "maps/b_bh10.bsp");
		self.noise = "items/r_item1.wav";
		self.healamount = 15;
		self.healtype = 0;
	}
	else
	if (self.spawnflags & H_MEGA)
	{
		precache_model("maps/b_bh100.bsp");
		precache_sound("items/r_item2.wav");
		setmodel(self, "maps/b_bh100.bsp");
		self.noise = "items/r_item2.wav";
		self.healamount = 100;
		self.healtype = 2;
	}
	else
	{
//		precache_model("maps/b_bh25.bsp");
		precache_model("progs/health_s.mdl");
		precache_sound("items/health1.wav");
//		setmodel(self, "maps/b_bh25.bsp");
		setmodel(self, "progs/health_s.mdl");
		self.noise = "items/health1.wav";
		self.healamount = 25;
		self.healtype = 1;
	}
	setsize (self, '-16 -16 0', '16 16 56');
	StartItem ();
};


void() health_touch =
{
	local	float amount;
	local	string	s;
	
	if (other.classname != "player")
		return;
	
	if (self.healtype == 2) // Megahealth?  Ignore max_health...
	{
		if (other.health >= 250)
			return;
		if (!T_Heal(other, self.healamount, 1))
			return;
	}
	else
	{
		if (!T_Heal(other, self.healamount, 0))
			return;
	}
	
	sprint(other, "You receive ");
	s = ftos(self.healamount);
	sprint(other, s);
	sprint(other, " health\n");
	
// health touch sound
	sound(other, CHAN_ITEM, self.noise, 1, ATTN_NORM);

//	stuffcmd (other, "bf\n");
	
	self.model = string_null;
	self.solid = SOLID_NOT;

	// Megahealth = rot down the player's super health
	if (self.healtype == 2)
	{
		other.items2 = other.items2 | IT2_SUPERHEALTH;
		self.nextthink = time + 5;
		self.think = item_megahealth_rot;
		self.owner = other;
	}
	else
	{
		if (deathmatch & DM_RESPAWN)		// deathmatch 2 is the silly old rules
		{
			if (deathmatch & DM_DM_MASK)
				self.nextthink = time + 20;
			self.think = SUB_regen;
		}
	}
	
	activator = other;
	SUB_UseTargets();				// fire all targets / killtargets
};	

void() item_megahealth_rot =
{
	other = self.owner;
	
	if (other.health > other.max_health)
	{
		other.health = other.health - 1;
		self.nextthink = time + 1;
		return;
	}

// it is possible for a player to die and respawn between rots, so don't
// just blindly subtract the flag off
	other.items2 = other.items2 - (other.items2 & IT2_SUPERHEALTH);
	
	if (deathmatch & DM_RESPAWN)	// deathmatch 2 is silly old rules
	{
		self.nextthink = time + 20;
		self.think = SUB_regen;
	}
};

/*
===============================================================================

ARMOR

===============================================================================
*/

void() armor_touch;

void() armor_touch =
{
	local	float	type, value, bit;
	
	if (other.health <= 0)
		return;
	if (other.classname != "player")
		return;

	if (self.classname == "item_armor1")
	{
		type = 1.0;
		value = 100;
		bit = IT2_ARMOR1;
	}
	if (self.classname == "item_armor2")
	{
		type = 1.0;
		value = 150;
		bit = IT2_ARMOR2;
	}
	if (self.classname == "item_armorInv")
	{
		type = 1.0;
		value = 200;
		bit = IT2_ARMOR3;
	}
	if (other.armortype*other.armorvalue >= type*value)
		return;

	other.armortype = type;
	other.armorvalue = value;
	other.items2 = other.items2 - (other.items2 & (IT2_ARMOR1 | IT2_ARMOR2 | IT2_ARMOR3)) + bit;

	self.solid = SOLID_NOT;
	self.model = string_null;
	if (deathmatch & DM_RESPAWN)
		self.nextthink = time + 20;
	self.think = SUB_regen;

	sprint(other, "You got armor\n");
// armor touch sound
	sound(other, CHAN_ITEM, "items/armor1.wav", 1, ATTN_NORM);
//	stuffcmd (other, "bf\n");
	
	activator = other;
	SUB_UseTargets();				// fire all targets / killtargets
};


/*QUAKED item_armor1 (0 .5 .8) (-16 -16 0) (16 16 32)
*/

void() item_armor1 =
{
	if (deathmatch & DM_NO_ARMOR)
		return;

	self.touch = armor_touch;
	precache_model ("progs/armor.mdl");
	setmodel (self, "progs/armor.mdl");
	self.skin = 0;
	setsize (self, '-16 -16 0', '16 16 56');
	StartItem ();
};

/*QUAKED item_armor2 (0 .5 .8) (-16 -16 0) (16 16 32)
*/

void() item_armor2 =
{
	if (deathmatch & DM_NO_ARMOR)
		return;

	self.touch = armor_touch;
	precache_model ("progs/armor.mdl");
	setmodel (self, "progs/armor.mdl");
	self.skin = 1;
	setsize (self, '-16 -16 0', '16 16 56');
	StartItem ();
};

/*QUAKED item_armorInv (0 .5 .8) (-16 -16 0) (16 16 32)
*/

void() item_armorInv =
{
	if (deathmatch & DM_NO_ARMOR)
		return;

	self.touch = armor_touch;
	precache_model ("progs/armor.mdl");
	setmodel (self, "progs/armor.mdl");
	self.skin = 2;
	setsize (self, '-16 -16 0', '16 16 56');
	StartItem ();
};

/*
===============================================================================

WEAPONS

===============================================================================
*/

void() bound_other_ammo =
{
	if (other.ammo_shells1 > 25)
		other.ammo_shells1 = 25;
	if (other.ammo_nails1 > 250)
		other.ammo_nails1 = 250;
	if (other.ammo_rockets1 > 30)
		other.ammo_rockets1 = 30;
	if (other.ammo_cells1 > 15)
		other.ammo_cells1 = 15;
	if (other.ammo_50cal > 20)
		other.ammo_50cal = 20;
};


float(entity ent, float w) RankForWeapon =
{
	if (w == IT_LIGHTNING)
		return 1;
	if (w == IT_ROCKET_LAUNCHER)
		return 2;
	if (w == IT_GRENADE_LAUNCHER)
		return 3;
	if (w == IT_SUPER_NAILGUN)
		return 4;
	if (w == IT_M99)
		return 5;
	if (w == IT_UZI)
		return 6;
	if (w == IT_NAILGUN)
		return 7;
	if (w == IT_SUPER_SHOTGUN)
		return 8;
	if (w == IT_SHOTGUN)
		return 9;
	if (w == IT_AXE)
		return 10;
	if (w == IT_TEKBOW)
		return 11;
	return 11;
};

/*
=============
Deathmatch_Weapon

Deathmatch weapon change rules for picking up a weapon

.float		ammo_shells1, ammo_nails1, ammo_rockets1, ammo_cells1;
=============
*/
void(entity ent, float new) Deathmatch_Weapon =
{
	local float or, nr;

// change self.weapon if desired
// 1998-08-15 Improved weapon switching on pickups by Maddes  start
//	or = RankForWeapon (self.weapon);
//	nr = RankForWeapon (new);
	or = RankForWeapon (ent, ent.weapon);
	nr = RankForWeapon (ent, new);
// 1998-08-15 Improved weapon switching on pickups by Maddes  end
	if ( nr < or )
// 1998-08-15 Improved weapon switching on pickups by Maddes  start
//		self.weapon = new;
		ent.weapon = new;
// 1998-08-15 Improved weapon switching on pickups by Maddes  end
};

/*
=============
weapon_touch
=============
*/
float() W_BestWeapon;

void() weapon_touch =
{
	local	float	hadammo, best, new, old;
	local	entity	stemp;
	local	float	leave;

	if (!(other.flags & FL_CLIENT))
		return;

// if the player was using his best weapon, change up to the new one if better		
	stemp = self;
	self = other;
	best = W_BestWeapon();
	self = stemp;

	if (deathmatch & DM_WEAPONS_STAY || coop)
		leave = 1;
	else
		leave = 0;

	if (self.classname == "weapon_shotgun")
	{
		if (leave && (other.items & IT_SHOTGUN) )
			return;
		hadammo = other.ammo_nails1;
		new = IT_SHOTGUN;
		other.ammo_nails1 = other.ammo_nails1 + 16;
	}
	else if (self.classname == "weapon_nailgun")
	{
		if (leave && (other.items & IT_NAILGUN) )
			return;
		hadammo = other.ammo_rockets1;
		new = IT_NAILGUN;
		other.ammo_nails1 = other.ammo_nails1 + 24;
	}

	else if (self.classname == "weapon_uzi")
	{
		if (leave && (other.items & IT_UZI) )
			return;
		hadammo = other.ammo_rockets1;
		new = IT_UZI;
		other.ammo_nails1 = other.ammo_nails1 + 32;
	}

	else if (self.classname == "weapon_sniper")
	{
		if (leave && (other.items & IT_M99) )
			return;
		hadammo = other.ammo_rockets1;
		new = IT_M99;
		other.ammo_50cal = other.ammo_50cal + 10;
	}

	else if (self.classname == "weapon_supernailgun")
	{
		if (leave && (other.items & IT_SUPER_NAILGUN) )
			return;
		hadammo = other.ammo_rockets1;
		new = IT_SUPER_NAILGUN;
		other.ammo_nails1 = other.ammo_nails1 + 50;
	}
	else if (self.classname == "weapon_supershotgun")
	{
		if (leave && (other.items & IT_SUPER_SHOTGUN) )
			return;
		hadammo = other.ammo_rockets1;
		new = IT_SUPER_SHOTGUN;
		other.ammo_shells1 = other.ammo_shells1 + 8;
	}
	else if (self.classname == "weapon_rocketlauncher")
	{
		if (leave && (other.items & IT_ROCKET_LAUNCHER) )
			return;
		hadammo = other.ammo_rockets1;
		new = IT_ROCKET_LAUNCHER;
		other.ammo_rockets1 = other.ammo_rockets1 + 4;
	}
	else if (self.classname == "weapon_grenadelauncher")
	{
		if (leave && (other.items & IT_GRENADE_LAUNCHER) )
			return;
		hadammo = other.ammo_rockets1;
		new = IT_GRENADE_LAUNCHER;
		other.ammo_rockets1 = other.ammo_rockets1 + 4;
	}
	else if (self.classname == "weapon_lightning")
	{
		if (leave && (other.items & IT_LIGHTNING) )
			return;
		hadammo = other.ammo_rockets1;
		new = IT_LIGHTNING;
		other.ammo_cells1 = other.ammo_cells1 + 5;
	}
	else
		objerror ("weapon_touch: unknown classname");

	sprint (other, "Picked up a ");
	sprint (other, self.netname);
	sprint (other, "\n");
// weapon touch sound
	sound (other, CHAN_ITEM, "weapons/pkup.wav", 1, ATTN_NORM);
//	stuffcmd (other, "bf\n");

	bound_other_ammo ();

// change to the weapon
// 1998-08-15 Improved weapon switching on pickups by Maddes  start
//	old = other.items;
// ...always if new one
//		if (!other.items & new)
//			Deathmatch_Weapon (other, new);
// 1998-08-15 Improved weapon switching on pickups by Maddes  end
	other.items = other.items | new;

	stemp = self;
	self = other;

//	if ( other.weapon == best )
//	   self.weapon = W_BestWeapon();

//    W_WeaponChange ();

	UpdateAmmoCounts ( self );

	if (self.zoomtoggle == 1)
		W_SetAmmoOnly();
	else
        W_SetCurrentAmmo();
	
	self = stemp;

	if (leave)
		return;

// remove it in single player, or setup for respawning in deathmatch
	self.model = string_null;
	self.solid = SOLID_NOT;
	if (deathmatch & DM_RESPAWN)
		self.nextthink = time + 30;
	self.think = SUB_regen;
	
	activator = other;
	SUB_UseTargets();				// fire all targets / killtargets
};

void() weapon_supershotgun;
void() weapon_nailgun;
void() weapon_uzi;
void() weapon_sniper;
void() weapon_supernailgun;
void() weapon_grenadelauncher;
void() weapon_rocketlauncher;
void() weapon_lightning;

/*QUAKED weapon_shotgun (0 .5 .8) (-16 -16 0) (16 16 32)
*/

void() weapon_shotgun =
{
	if (deathmatch & DM_AUTOMATICS)
	{
		self.classname = "weapon_uzi";
		weapon_uzi();
		return;
	}
	else if (deathmatch & DM_SHOTGUNS)
	{
		self.classname = "weapon_supershotgun";
		weapon_supershotgun();
		return;
	}
	else if (deathmatch & DM_EXPLOSIVES)
	{
		self.classname = "weapon_grenadelauncher";
		weapon_grenadelauncher();
		return;
	}
	else if (deathmatch & DM_SNIPERS)
	{
		self.classname = "weapon_sniper";
		weapon_sniper();
		return;
	}
	else
	{
		setmodel (self, "progs/g_pistol.mdl");
		self.weapon = IT_SHOTGUN;
		self.netname = "Pistol";
		self.classname = "weapon_shotgun";
		self.touch = weapon_touch;
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};

/*QUAKED weapon_supershotgun (0 .5 .8) (-16 -16 0) (16 16 32)
*/

void() weapon_supershotgun =
{
	if (deathmatch & DM_PISTOLS)
	{
		self.classname = "weapon_shotgun";
		weapon_shotgun();
		return;
	}
	else if (deathmatch & DM_AUTOMATICS)
	{
		self.classname = "weapon_uzi";
		weapon_uzi();
		return;
	}
	else if (deathmatch & DM_EXPLOSIVES)
	{
		self.classname = "weapon_lightning";
		weapon_lightning();
		return;
	}
	else if (deathmatch & DM_SNIPERS)
	{
		self.classname = "weapon_sniper";
		weapon_sniper();
		return;
	}
	else
	{
		setmodel (self, "progs/g_shot.mdl");
		self.weapon = IT_SUPER_SHOTGUN;
		self.netname = "Shotgun";
		self.touch = weapon_touch;
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};

/*QUAKED weapon_nailgun (0 .5 .8) (-16 -16 0) (16 16 32)
*/

void() weapon_nailgun =
{
	if (deathmatch & DM_PISTOLS)
	{
		self.classname = "weapon_shotgun";
		weapon_shotgun();
		return;
	}
	else if (deathmatch & DM_SHOTGUNS)
	{
		self.classname = "weapon_supershotgun";
		weapon_supershotgun();
		return;
	}
	else if (deathmatch & DM_EXPLOSIVES)
	{
		self.classname = "weapon_grenadelauncher";
		weapon_grenadelauncher();
		return;
	}
	else if (deathmatch & DM_SNIPERS)
	{
		self.classname = "weapon_sniper";
		weapon_sniper();
		return;
	}
	else
	{
		setmodel (self, "progs/g_nail.mdl");
		self.weapon = IT_NAILGUN;
		self.netname = "Assualt Rifle";
		self.touch = weapon_touch;
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};

/*QUAKED weapon_uzi (0 .5 .8) (-16 -16 0) (16 16 32)
*/

void() weapon_uzi =
{
	if (deathmatch & DM_PISTOLS)
	{
		self.classname = "weapon_shotgun";
		weapon_shotgun();
		return;
	}
	else if (deathmatch & DM_SHOTGUNS)
	{
		self.classname = "weapon_supershotgun";
		weapon_supershotgun();
		return;
	}
	else if (deathmatch & DM_EXPLOSIVES)
	{
		self.classname = "weapon_lightning";
		weapon_lightning();
		return;
	}
	else if (deathmatch & DM_SNIPERS)
	{
		self.classname = "weapon_sniper";
		weapon_sniper();
		return;
	}
	else
	{
		setmodel (self, "progs/g_uzi.mdl");
		self.weapon = IT_UZI;
		self.netname = "Uzi";
		self.touch = weapon_touch;
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};

/*QUAKED weapon_sniper (0 .5 .8) (-16 -16 0) (16 16 32)
*/

void() weapon_sniper =
{
	if (deathmatch & DM_PISTOLS)
	{
		self.classname = "weapon_shotgun";
		weapon_shotgun();
		return;
	}
	else if (deathmatch & DM_AUTOMATICS)
	{
		self.classname = "weapon_nailgun";
		weapon_nailgun();
		return;
	}
	else if (deathmatch & DM_SHOTGUNS)
	{
		self.classname = "weapon_supershotgun";
		weapon_supershotgun();
		return;
	}
	else if (deathmatch & DM_EXPLOSIVES)
	{
		self.classname = "weapon_rocketlauncher";
		weapon_rocketlauncher();
		return;
	}
	else
	{
		setmodel (self, "progs/g_sniper.mdl");
		self.weapon = IT_M99;
		self.netname = "Sniper Rifle";
		self.touch = weapon_touch;
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};

/*QUAKED weapon_supernailgun (0 .5 .8) (-16 -16 0) (16 16 32)
*/

void() weapon_supernailgun =
{
	if (deathmatch & DM_PISTOLS)
	{
		self.classname = "weapon_shotgun";
		weapon_shotgun();
		return;
	}
	else if (deathmatch & DM_SHOTGUNS)
	{
		self.classname = "weapon_supershotgun";
		weapon_supershotgun();
		return;
	}
	else if (deathmatch & DM_EXPLOSIVES)
	{
		self.classname = "weapon_rocketlauncher";
		weapon_rocketlauncher();
		return;
	}
	else if (deathmatch & DM_SNIPERS)
	{
		self.classname = "weapon_sniper";
		weapon_sniper();
		return;
	}
	else
	{
		setmodel (self, "progs/g_nail2.mdl");
		self.weapon = IT_SUPER_NAILGUN;
		self.netname = "Mini Gun";
		self.touch = weapon_touch;
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};

/*QUAKED weapon_grenadelauncher (0 .5 .8) (-16 -16 0) (16 16 32)
*/

void() weapon_grenadelauncher =
{
	if (deathmatch & DM_PISTOLS)
	{
		self.classname = "weapon_shotgun";
		weapon_shotgun();
		return;
	}
	else if (deathmatch & DM_AUTOMATICS)
	{
        self.classname = "weapon_uzi";
		weapon_uzi();
		return;
	}
	else if (deathmatch & DM_SHOTGUNS)
	{
		self.classname = "weapon_supershotgun";
		weapon_supershotgun();
		return;
	}
	else if (deathmatch & DM_SNIPERS)
	{
		self.classname = "weapon_sniper";
		weapon_sniper();
		return;
	}
	else
	{
		setmodel (self, "progs/g_rock.mdl");
		self.weapon = IT_GRENADE_LAUNCHER;
		self.netname = "Grenade Launcher";
		self.touch = weapon_touch;
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};

/*QUAKED weapon_rocketlauncher (0 .5 .8) (-16 -16 0) (16 16 32)
*/

void() weapon_rocketlauncher =
{
	if (deathmatch & DM_PISTOLS)
	{
		self.classname = "weapon_shotgun";
		weapon_shotgun();
		return;
	}
	else if (deathmatch & DM_AUTOMATICS)
	{
		self.classname = "weapon_nailgun";
		weapon_nailgun();
		return;
	}
	else if (deathmatch & DM_SHOTGUNS)
	{
		self.classname = "weapon_supershotgun";
		weapon_supershotgun();
        return;
	}
	else if (deathmatch & DM_SNIPERS)
	{
		self.classname = "weapon_sniper";
		weapon_sniper();
		return;
	}
	else
	{
		setmodel (self, "progs/g_rock2.mdl");
		self.weapon = IT_ROCKET_LAUNCHER;
		self.netname = "Rocket Launcher";
		self.touch = weapon_touch;
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};


/*QUAKED weapon_lightning (0 .5 .8) (-16 -16 0) (16 16 32)
*/

void() weapon_lightning =
{
	if (deathmatch & DM_PISTOLS)
	{
		self.classname = "weapon_shotgun";
		weapon_shotgun();
		return;
	}
	else if (deathmatch & DM_AUTOMATICS)
	{
		self.classname = "weapon_supernailgun";
		weapon_supernailgun();
		return;
	}
	else if (deathmatch & DM_SHOTGUNS)
	{
		self.classname = "weapon_supershotgun";
		weapon_supershotgun();
		return;
	}
	else if (deathmatch & DM_SNIPERS)
	{
		self.classname = "weapon_sniper";
		weapon_sniper();
		return;
	}
	else
	{
		setmodel (self, "progs/g_light.mdl");
		self.weapon = IT_LIGHTNING;
		self.netname = "set of Remote Mines";
		self.touch = weapon_touch;
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};


/*
===============================================================================

AMMO

===============================================================================
*/

void() ammo_touch =
{
       local entity	stemp;
       local float		best;

	if (other.classname != "player")
		return;
	if (other.health <= 0)
		return;

// if the player was using his best weapon, change up to the new one if better		
	stemp = self;
	self = other;
	best = W_BestWeapon();
	self = stemp;


// shotgun
	if (self.weapon == 1)
	{
		if (other.ammo_shells1 >= 25)
			return;
		other.ammo_shells1 = other.ammo_shells1 + self.aflag;
	}

// spikes
	if (self.weapon == 2)
	{
		if (other.ammo_nails1 >= 250)
			return;
		other.ammo_nails1 = other.ammo_nails1 + self.aflag;
	}

//	rockets
	if (self.weapon == 3)
	{
		if (other.ammo_rockets1 >= 30)
			return;
		other.ammo_rockets1 = other.ammo_rockets1 + self.aflag;
	}

//	cells
	if (self.weapon == 4)
	{
		if (other.ammo_cells1 >= 15)
			return;
		other.ammo_cells1 = other.ammo_cells1 + self.aflag;
	}

// 50 caliber
	if (self.weapon == 5)
	{
		if (other.ammo_50cal >= 20)
			return;
		other.ammo_50cal = other.ammo_50cal + self.aflag;
	}

	UpdateAmmoCounts ( other );

	bound_other_ammo ();
	
	sprint (other, "Picked up some ");
	sprint (other, self.netname);
	sprint (other, "\n");
// ammo touch sound
	sound (other, CHAN_ITEM, "weapons/pkup.wav", 1, ATTN_NORM);
//	stuffcmd (other, "bf\n");
/*
// change to a better weapon if appropriate

// 1998-08-15 Improved weapon switching on pickups by Maddes  start
	stemp = self;
	self = other;
// 1998-08-15 Improved weapon switching on pickups by Maddes  end

	if ( other.weapon == best )
	{
// 1998-08-15 Improved weapon switching on pickups by Maddes  start
//		stemp = self;
//		self = other;
// 1998-08-15 Improved weapon switching on pickups by Maddes  end
		self.weapon = W_BestWeapon();
//	    W_WeaponChange ();
//      W_SetCurrentAmmo();
//		self = stemp;
	}
*/

// if changed current ammo, update it

	stemp = self;
	self = other;
//	W_WeaponChange ();
		if (self.zoomtoggle == 1)
			W_SetAmmoOnly();
		else
            W_SetCurrentAmmo();
	self = stemp;

// remove it in single player, or setup for respawning in deathmatch
	self.model = string_null;
	self.solid = SOLID_NOT;
	if (deathmatch & DM_RESPAWN)
		self.nextthink = time + 30;
	self.think = SUB_regen;

	activator = other;
	SUB_UseTargets();				// fire all targets / killtargets
};

void() item_shells;
void() item_spikes;
void() item_rockets;
void() item_cells;
void() item_50cal;

float WEAPON_BIG2 = 1;

/*QUAKED item_shells (0 .5 .8) (0 0 0) (32 32 32) big
*/

void() item_shells =
{
	if ((deathmatch & DM_AUTOMATICS) || (deathmatch & DM_PISTOLS))
	{
		item_spikes();
		return;
	}
	else if (deathmatch & DM_EXPLOSIVES)
	{
		// Make it a weapon pickup instead to reduce confusion.
		self.classname = "weapon_lightning";
		weapon_lightning();
//		item_cells();
		return;
	}
	else if (deathmatch & DM_SNIPERS)
	{
		item_50cal();
		return;
	}
	else
	{
		self.touch = ammo_touch;

		if (self.spawnflags & WEAPON_BIG2)
		{
			precache_model ("progs/s_box_s.mdl");
			setmodel (self, "progs/s_box_s.mdl");
			self.aflag = 25;
		}
		else
		{
			precache_model ("progs/s_box_s.mdl");
			setmodel (self, "progs/s_box_s.mdl");
			self.aflag = 10;
		}
		self.weapon = 1;
		self.netname = "shells";
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};

/*QUAKED item_spikes (0 .5 .8) (0 0 0) (32 32 32) big
*/

void() item_spikes =
{
	if (deathmatch & DM_SHOTGUNS)
	{
		item_shells();
		return;
	}
	else if (deathmatch & DM_EXPLOSIVES)
	{
		item_rockets();
		return;
	}
	else if (deathmatch & DM_SNIPERS)
	{
		item_50cal();
		return;
	}
	else
	{
		self.touch = ammo_touch;

		if (self.spawnflags & WEAPON_BIG2)
		{
	//		precache_model ("maps/b_nail1.bsp");
	//		setmodel (self, "maps/b_nail1.bsp");
			precache_model ("progs/b_box_s.mdl");
			setmodel (self, "progs/b_box_s.mdl");
			self.aflag = 64;
		}
		else
		{
	//		precache_model ("maps/b_nail0.bsp");
	//		setmodel (self, "maps/b_nail0.bsp");
			precache_model ("progs/b_box_s.mdl");
			setmodel (self, "progs/b_box_s.mdl");
			self.aflag = 32;
		}
		self.weapon = 2;
		self.netname = "bullets";
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};

/*QUAKED item_rockets (0 .5 .8) (0 0 0) (32 32 32) big
*/

void() item_rockets =
{
	if ((deathmatch & DM_AUTOMATICS) || (deathmatch & DM_PISTOLS))
	{
		item_spikes();
		return;
	}
	else if (deathmatch & DM_SHOTGUNS)
	{
		item_shells();
		return;
	}
	else if (deathmatch & DM_SNIPERS)
	{
		item_50cal();
		return;
	}
	else
	{
		self.touch = ammo_touch;

		if (self.spawnflags & WEAPON_BIG2)
		{
	//		precache_model ("maps/b_rock1.bsp");
	//		setmodel (self, "maps/b_rock1.bsp");
			precache_model ("progs/e_box_s.mdl");
			setmodel (self, "progs/e_box_s.mdl");
			self.aflag = 8;
		}
		else
		{
	//		precache_model ("maps/b_rock0.bsp");
	//		setmodel (self, "maps/b_rock0.bsp");
			precache_model ("progs/e_box_s.mdl");
			setmodel (self, "progs/e_box_s.mdl");
			self.aflag = 4;
		}
		self.weapon = 3;
		self.netname = "missiles";
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};


/*QUAKED item_cells (0 .5 .8) (0 0 0) (32 32 32) big
*/

void() item_cells =
{
	if ((deathmatch & DM_AUTOMATICS) || (deathmatch & DM_PISTOLS))
	{
		item_spikes();
		return;
	}
	else if (deathmatch & DM_SHOTGUNS)
	{
		item_shells();
		return;
	}
	else if (deathmatch & DM_SNIPERS)
	{
		item_50cal();
		return;
	}
	else
	{
		self.touch = ammo_touch;

		if (self.spawnflags & WEAPON_BIG2)
		{
	//		precache_model ("maps/b_batt1.bsp");
	//		setmodel (self, "maps/b_batt1.bsp");
	//		precache_model ("progs/c_box_s.mdl");
	//		setmodel (self, "progs/c_box_s.mdl");
		    setmodel (self, "progs/g_light.mdl");
			self.aflag = 15;
		}
		else
		{
	//		precache_model ("maps/b_batt0.bsp");
	//		setmodel (self, "maps/b_batt0.bsp");
	//		precache_model ("progs/c_box_s.mdl");
	//		setmodel (self, "progs/c_box_s.mdl");
		    setmodel (self, "progs/g_light.mdl");
			self.aflag = 5;
		}
		self.weapon = 4;
		self.netname = "remote mines";
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};

void() item_50cal =
{
	if ((deathmatch & DM_AUTOMATICS) || (deathmatch & DM_PISTOLS))
	{
		item_spikes();
		return;
	}
	else if (deathmatch & DM_SHOTGUNS)
	{
		item_shells();
		return;
	}
	else if (deathmatch & DM_EXPLOSIVES)
	{
		item_rockets();
		return;
	}
	else
	{
		self.touch = ammo_touch;

		if (self.spawnflags & WEAPON_BIG2)
		{
	//		precache_model ("maps/b_nail1.bsp");
	//		setmodel (self, "maps/b_nail1.bsp");
			precache_model ("progs/5_box_s.mdl");
			setmodel (self, "progs/5_box_s.mdl");
			self.aflag = 20;
		}
		else
		{
	//		precache_model ("maps/b_nail0.bsp");
	//		setmodel (self, "maps/b_nail0.bsp");
			precache_model ("progs/5_box_s.mdl");
			setmodel (self, "progs/5_box_s.mdl");
			self.aflag = 10;
		}
		self.weapon = 5;
		self.netname = "50.cal rounds";
		setsize (self, '-16 -16 0', '16 16 56');
		StartItem ();
	}
};

/*QUAKED item_weapon (0 .5 .8) (0 0 0) (32 32 32) shotgun rocket spikes big
DO NOT USE THIS!!!! IT WILL BE REMOVED!
*/

float WEAPON_SHOTGUN = 1;
float WEAPON_ROCKET = 2;
float WEAPON_SPIKES = 4;
float WEAPON_BIG = 8;
void() item_weapon =
{
	self.touch = ammo_touch;

	if (self.spawnflags & WEAPON_SHOTGUN)
	{
		if (self.spawnflags & WEAPON_BIG)
		{
//			precache_model ("maps/b_shell1.bsp");
//			setmodel (self, "maps/b_shell1.bsp");
            precache_model ("progs/s_box_s.mdl");
            setmodel (self, "progs/s_box_s.mdl");
			self.aflag = 40;
		}
		else
		{
//			precache_model ("maps/b_shell0.bsp");
//			setmodel (self, "maps/b_shell0.bsp");
            precache_model ("progs/s_box_s.mdl");
            setmodel (self, "progs/s_box_s.mdl");
			self.aflag = 20;
		}
		self.weapon = 1;
		self.netname = "shells";
	}

	if (self.spawnflags & WEAPON_SPIKES)
	{
		if (self.spawnflags & WEAPON_BIG)
		{
//			precache_model ("maps/b_nail1.bsp");
//			setmodel (self, "maps/b_nail1.bsp");
            precache_model ("progs/b_box_s.mdl");
            setmodel (self, "progs/b_box_s.mdl");
			self.aflag = 40;
		}
		else
		{
//			precache_model ("maps/b_nail0.bsp");
//			setmodel (self, "maps/b_nail0.bsp");
            precache_model ("progs/b_box_s.mdl");
            setmodel (self, "progs/b_box_s.mdl");
			self.aflag = 20;
		}
		self.weapon = 2;
		self.netname = "bullets";
	}

	if (self.spawnflags & WEAPON_ROCKET)
	{
		if (self.spawnflags & WEAPON_BIG)
		{
//			precache_model ("maps/b_rock1.bsp");
//			setmodel (self, "maps/b_rock1.bsp");
            precache_model ("progs/e_box_s.mdl");
            setmodel (self, "progs/e_box_s.mdl");
			self.aflag = 10;
		}
		else
		{
//			precache_model ("maps/b_rock0.bsp");
//			setmodel (self, "maps/b_rock0.bsp");
            precache_model ("progs/e_box_s.mdl");
            setmodel (self, "progs/e_box_s.mdl");
			self.aflag = 5;
		}
		self.weapon = 3;
		self.netname = "rockets";
	}
	
	setsize (self, '0 0 0', '32 32 56');
	StartItem ();
};


/*
===============================================================================

KEYS

===============================================================================
*/

void() key_touch =
{
local entity	stemp;
local float		best;

	if (other.classname != "player")
		return;
	if (other.health <= 0)
		return;
	if (other.items & self.items)
		return;

	sprint (other, "You got the ");
	sprint (other, self.netname);
	sprint (other,"\n");

	sound (other, CHAN_ITEM, self.noise, 1, ATTN_NORM);
//	stuffcmd (other, "bf\n");
	other.items = other.items | self.items;

	if (!coop)
	{
		self.solid = SOLID_NOT;
		self.model = string_null;
	}

	activator = other;
	SUB_UseTargets();				// fire all targets / killtargets
};


void() key_setsounds =
{
	if (world.worldtype == 0)
	{
		precache_sound ("misc/medkey.wav");
		self.noise = "misc/medkey.wav";
	}
	if (world.worldtype == 1)
	{
		precache_sound ("misc/runekey.wav");
		self.noise = "misc/runekey.wav";
	}
	if (world.worldtype == 2)
	{
		precache_sound2 ("misc/basekey.wav");
		self.noise = "misc/basekey.wav";
	}
};

/*QUAKED item_key1 (0 .5 .8) (-16 -16 -24) (16 16 32)
SILVER key
In order for keys to work
you MUST set your maps
worldtype to one of the
following:
0: medieval
1: metal
2: base
*/

void() item_key1 =
{
	if (world.worldtype == 0)
	{
		precache_model ("progs/w_s_key.mdl");
		setmodel (self, "progs/w_s_key.mdl");
		self.netname = "silver key";
	}
	else if (world.worldtype == 1)
	{
		precache_model ("progs/m_s_key.mdl");
		setmodel (self, "progs/m_s_key.mdl");
		self.netname = "silver runekey";
	}
	else if (world.worldtype == 2)
	{
		precache_model2 ("progs/b_s_key.mdl");
		setmodel (self, "progs/b_s_key.mdl");
		self.netname = "silver keycard";
	}
	key_setsounds();
	self.touch = key_touch;
	self.items = IT_KEY1;
	setsize (self, '-16 -16 -24', '16 16 32');
	StartItem ();
};

/*QUAKED item_key2 (0 .5 .8) (-16 -16 -24) (16 16 32)
GOLD key
In order for keys to work
you MUST set your maps
worldtype to one of the
following:
0: medieval
1: metal
2: base
*/

void() item_key2 =
{
	if (world.worldtype == 0)
	{
		precache_model ("progs/w_g_key.mdl");
		setmodel (self, "progs/w_g_key.mdl");
		self.netname = "gold key";
	}
	if (world.worldtype == 1)
	{
		precache_model ("progs/m_g_key.mdl");
		setmodel (self, "progs/m_g_key.mdl");
		self.netname = "gold runekey";
	}
	if (world.worldtype == 2)
	{
		precache_model2 ("progs/b_g_key.mdl");
		setmodel (self, "progs/b_g_key.mdl");
		self.netname = "gold keycard";
	}
	key_setsounds();
	self.touch = key_touch;
	self.items = IT_KEY2;
	setsize (self, '-16 -16 -24', '16 16 32');
	StartItem ();
};



/*
===============================================================================

END OF LEVEL RUNES

===============================================================================
*/

void() sigil_touch =
{
local entity	stemp;
local float		best;

	if (other.classname != "player")
		return;
	if (other.health <= 0)
		return;

	centerprint (other, "You got the rune!");

	sound (other, CHAN_ITEM, self.noise, 1, ATTN_NORM);
//	stuffcmd (other, "bf\n");
	self.solid = SOLID_NOT;
	self.model = string_null;
	serverflags = serverflags | (self.spawnflags & 15);
	self.classname = "";		// so rune doors won't find it
	
	activator = other;
	SUB_UseTargets();				// fire all targets / killtargets
};


/*QUAKED item_sigil (0 .5 .8) (-16 -16 -24) (16 16 32) E1 E2 E3 E4
End of level sigil, pick up to end episode and return to jrstart.
*/

void() item_sigil =
{
	if (!self.spawnflags)
		objerror ("no spawnflags");

	precache_sound ("misc/runekey.wav");
	self.noise = "misc/runekey.wav";

	if (self.spawnflags & 1)
	{
		precache_model ("progs/end1.mdl");
		setmodel (self, "progs/end1.mdl");
	}
	if (self.spawnflags & 2)
	{
		precache_model2 ("progs/end2.mdl");
		setmodel (self, "progs/end2.mdl");
	}
	if (self.spawnflags & 4)
	{
		precache_model2 ("progs/end3.mdl");
		setmodel (self, "progs/end3.mdl");
	}
	if (self.spawnflags & 8)
	{
		precache_model2 ("progs/end4.mdl");
		setmodel (self, "progs/end4.mdl");
	}
	
	self.touch = sigil_touch;
	setsize (self, '-16 -16 -24', '16 16 32');
	StartItem ();
};

/*
===============================================================================

POWERUPS

===============================================================================
*/

void() spawn_protect_off =
{
	if (self.aqstate & AQS_SPAWN_PROTECT)
	{
		self.aqstate = self.aqstate - AQS_SPAWN_PROTECT;
		self.invincible_time = 0;
		self.invincible_finished = 0;
		self.takedamage = DAMAGE_AIM;
		self.effects = self.effects - (self.effects & EF_REDLIGHT);
	}
};

void(entity e) spawn_protect_off_e  =
{
	local entity old_self;

	old_self = self;
	self = e;
	spawn_protect_off ();
	self = old_self;
};

void() powerup_touch;


void() powerup_touch =
{
local entity	stemp;
local float		best;

	if (other.classname != "player")
		return;
	if (other.health <= 0)
		return;

	sprint (other, "You got the ");
	sprint (other, self.netname);
	sprint (other,"\n");

	if (deathmatch & DM_DM_MASK)
	{
		self.mdl = self.model;
		
		if ((self.classname == "item_artifact_invulnerability") ||
			(self.classname == "item_artifact_invisibility"))
			self.nextthink = time + 60*5;
		else
			self.nextthink = time + 60;
		
		self.think = SUB_regen;
	}

	spawn_protect_off_e (other);

	sound (other, CHAN_VOICE, self.noise, 1, ATTN_NORM);
//	stuffcmd (other, "bf\n");
	self.solid = SOLID_NOT;
	other.items = other.items | self.items;
	self.model = string_null;

// do the apropriate action
	if (self.classname == "item_artifact_envirosuit")
	{
		other.rad_time = 1;
		other.radsuit_finished = time + 30;
	}
	
	if (self.classname == "item_artifact_invulnerability")
	{
		other.invincible_time = 1;
		other.invincible_finished = time + 30;
	}
	
	if (self.classname == "item_artifact_invisibility")
	{
		other.invisible_time = 1;
		other.invisible_finished = time + 30;
	}

	if (self.classname == "item_artifact_super_damage")
	{
		other.super_time = 1;
		other.super_damage_finished = time + 30;
	}	

	activator = other;
	SUB_UseTargets();				// fire all targets / killtargets
};



/*QUAKED item_artifact_invulnerability (0 .5 .8) (-16 -16 -24) (16 16 32)
Player is invulnerable for 30 seconds
*/
void() item_artifact_invulnerability =
{
	self.touch = powerup_touch;

	precache_model ("progs/invulner.mdl");
	precache_sound ("items/protect.wav");
	precache_sound ("items/protect2.wav");
	precache_sound ("items/protect3.wav");
	self.noise = "items/protect.wav";
	setmodel (self, "progs/invulner.mdl");
	self.netname = "Talisman of Protection";
	self.items = IT_INVULNERABILITY;
	setsize (self, '-16 -16 -24', '16 16 32');
	StartItem ();
};

/*QUAKED item_artifact_envirosuit (0 .5 .8) (-16 -16 -24) (16 16 32)
Player takes no damage from water or slime for 30 seconds
*/
void() item_artifact_envirosuit =
{
	self.touch = powerup_touch;

	precache_model ("progs/suit.mdl");
	precache_sound ("items/suit.wav");
	precache_sound ("items/suit2.wav");
	self.noise = "items/suit.wav";
	setmodel (self, "progs/suit.mdl");
	self.netname = "Talisman of Nature";
	self.items = IT_SUIT;
	setsize (self, '-16 -16 -24', '16 16 32');
	StartItem ();
};


/*QUAKED item_artifact_invisibility (0 .5 .8) (-16 -16 -24) (16 16 32)
Player is invisible for 30 seconds
*/
void() item_artifact_invisibility =
{
	self.touch = powerup_touch;

	precache_model ("progs/invisibl.mdl");
	precache_sound ("items/inv1.wav");
	precache_sound ("items/inv2.wav");
	precache_sound ("items/inv3.wav");
	self.noise = "items/inv1.wav";
	setmodel (self, "progs/invisibl.mdl");
	self.netname = "Cloaking Gear";
	self.items = IT_INVISIBILITY;
	setsize (self, '-16 -16 0', '16 16 56');
	StartItem ();
};


/*QUAKED item_artifact_super_damage (0 .5 .8) (-16 -16 -24) (16 16 32)
The next attack from the player will do 4x damage
*/
void() item_artifact_super_damage =
{
	self.touch = powerup_touch;

	precache_model ("progs/quaddama.mdl");
	precache_sound ("items/damage.wav");
	precache_sound ("items/damage2.wav");
	precache_sound ("items/damage3.wav");
	self.noise = "items/damage.wav";
	setmodel (self, "progs/quaddama.mdl");
	self.netname = "Talisman of Power";
	self.items = IT_QUAD;
	setsize (self, '-16 -16 -24', '16 16 32');
	StartItem ();
};



/*
===============================================================================

PLAYER WEAPON DROP

===============================================================================
*/
void() BackpackTouch =
{
	local string	s;
	local	float	best, old, new;
	local		entity	stemp;
	local	float	acount;
	
	if (other.classname != "player")
		return;
	if (other.health <= 0)
		return;

	acount = 0;
	sprint (other, "Picked up ");

	if (self.items)
		if ((other.items & self.items) == 0)
		{
			acount = 1;
			sprint (other, "the ");
			sprint (other, self.netname);
		}

// if the player was using his best weapon, change up to the new one if better		
	stemp = self;
	self = other;
	best = W_BestWeapon();
	self = stemp;

// change weapons
	other.ammo_shells1 = other.ammo_shells1 + self.ammo_shells1;
	other.ammo_nails1 = other.ammo_nails1 + self.ammo_nails1;
	other.ammo_rockets1 = other.ammo_rockets1 + self.ammo_rockets1;
	other.ammo_cells1 = other.ammo_cells1 + self.ammo_cells1;

	other.ammo_50cal = other.ammo_50cal + self.ammo_50cal;

	new = self.items;
	if (!new)
		new = other.weapon;
// 1998-08-15 Improved weapon switching on pickups by Maddes  start
//	old = other.items;
// ...always if new one
	if (!other.items & new)
		Deathmatch_Weapon (other, new);
// 1998-08-15 Improved weapon switching on pickups by Maddes  end
	other.items = other.items | new;
	
	bound_other_ammo ();

	if (self.ammo_shells1)
	{
		if (acount)
			sprint(other, ", ");
		acount = 1;
		s = ftos(self.ammo_shells1);
		sprint (other, s);
		sprint (other, " shells");
	}
	if (self.ammo_nails1)
	{
		if (acount)
			sprint(other, ", ");
		acount = 1;
		s = ftos(self.ammo_nails1);
		sprint (other, s);
		sprint (other, " bullets");
	}
	if (self.ammo_rockets1)
	{
		if (acount)
			sprint(other, ", ");
		acount = 1;
		s = ftos(self.ammo_rockets1);
		sprint (other, s);
		sprint (other, " missiles");
	}
	if (self.ammo_cells1)
	{
		if (acount)
			sprint(other, ", ");
		acount = 1;
		s = ftos(self.ammo_cells1);
		sprint (other, s);
		sprint (other, " remote Mines");
	}
	if (self.ammo_50cal)
	{
		if (acount)
			sprint(other, ", ");
		acount = 1;
		s = ftos(self.ammo_50cal);
		sprint (other, s);
		sprint (other, " 50.cal rounds");
	}

	sprint (other, "\n");
// backpack touch sound
	sound (other, CHAN_ITEM, "weapons/pkup.wav", 1, ATTN_NORM);
//	stuffcmd (other, "bf\n");

// remove the backpack, change self to the player
	remove(self);
	self = other;

// change to the weapon
/*
    if (self.button0)
        W_SetCurrentAmmo();
    else if (self.weapon == IT_SHOTGUN)
	    W_SetCurrentAmmo();
    else if (self.items & IT_NAILGUN)
	    W_SetCurrentAmmo();
    else if (self.items & IT_UZI)
	    W_SetCurrentAmmo();
    else if (self.items & IT_SUPER_NAILGUN)
	    W_SetCurrentAmmo();
    else if (self.items & IT_SUPER_SHOTGUN)
	    W_SetCurrentAmmo();
    else if (self.items & IT_GRENADE_LAUNCHER)
	    W_SetCurrentAmmo();
    else if (self.items & IT_ROCKET_LAUNCHER)
	    W_SetCurrentAmmo();
    else if (self.items & IT_LIGHTNING)
	    W_SetCurrentAmmo();
    else
        {*/
//           self.weapon = W_BestWeapon();
	UpdateAmmoCounts (self);

		if (self.zoomtoggle == 1)
			W_SetAmmoOnly();
		else
            W_SetCurrentAmmo();

//	       W_WeaponChange();
//        }
};

/*
===============
DropBackpack
===============
*/
void() DropBackpack =
{
	local entity	item;

	// this is to compensate for the new way we're hacking the ammo
	// counts. If they're monsters, just copy shells->shells1 etc.
	if (self.flags & FL_MONSTER)
	{
		self.ammo_shells1 = self.ammo_shells;
		self.ammo_nails1 = self.ammo_nails;
		self.ammo_rockets1 = self.ammo_rockets;
		self.ammo_cells1 = self.ammo_cells;
	}

	if (!(self.ammo_shells1 + self.ammo_nails1 + self.ammo_rockets1 + self.ammo_cells1 + self.ammo_50cal))
		return;	// nothing in it

	item = spawn();
	item.origin = self.origin - '0 0 24';
	
	item.items = self.weapon;
	if (item.items == IT_AXE)
		item.netname = "Knife";
	else if (item.items == IT_TEKBOW)
		item.netname = "Tek Bow";
	else if (item.items == IT_SHOTGUN)
		item.netname = "Pistol";
	else if (item.items == IT_SUPER_SHOTGUN)
		item.netname = "Shotgun";
	else if (item.items == IT_NAILGUN)
		item.netname = "Assualt Rifle";
	else if (item.items == IT_UZI)
		item.netname = "Uzi";
	else if (item.items == IT_M99)
		item.netname = "Sniper Rifle";
	else if (item.items == IT_SUPER_NAILGUN)
		item.netname = "Mini Gun";
	else if (item.items == IT_GRENADE_LAUNCHER)
		item.netname = "Grenade Launcher";
	else if (item.items == IT_ROCKET_LAUNCHER)
		item.netname = "Rocket Launcher";
	else if (item.items == IT_LIGHTNING)
		item.netname = "Remote Mines";
	else
		item.netname = "";

	item.ammo_shells1 = self.ammo_shells1;
	item.ammo_nails1 = self.ammo_nails1;
	item.ammo_rockets1 = self.ammo_rockets1;
	item.ammo_cells1 = self.ammo_cells1;

	item.ammo_50cal = self.ammo_50cal;

	item.velocity_z = 300;
	item.velocity_x = -100 + (random() * 200);
	item.velocity_y = -100 + (random() * 200);
	
	item.flags = FL_ITEM;
	item.solid = SOLID_TRIGGER;
	item.movetype = MOVETYPE_TOSS;
	setmodel (item, "progs/backpack.mdl");
	setsize (item, '-16 -16 0', '16 16 56');
	item.touch = BackpackTouch;
	
	item.nextthink = time + 120;	// remove after 2 minutes
	item.think = SUB_Remove;
};

void() WeaponDropTouch =
{
	local string	s;
	local	float	best, old, new;
	local		entity	stemp;
	local	float	acount;
	
	if (other.classname != "player")
		return;
	if (other.health <= 0)
		return;

	acount = 0;
	sprint (other, "Picked up ");
	
	if (self.items)
		if ((other.items & self.items) == 0)
		{
			acount = 1;
			sprint (other, "a ");
			sprint (other, self.netname);
		}
/*
// if the player was using his best weapon, change up to the new one if better		
	stemp = self;
	self = other;
	best = W_BestWeapon();
	self = stemp;
*/
// change weapons
	other.ammo_shells1 = other.ammo_shells1 + self.ammo_shells1;
	other.ammo_nails1 = other.ammo_nails1 + self.ammo_nails1;
	other.ammo_rockets1 = other.ammo_rockets1 + self.ammo_rockets1;
	other.ammo_cells1 = other.ammo_cells1 + self.ammo_cells1;

	other.ammo_50cal = other.ammo_50cal + self.ammo_50cal;

	new = self.items;
	if (!new)
		new = other.weapon;
/*
// 1998-08-15 Improved weapon switching on pickups by Maddes  start
//	old = other.items;
// ...always if new one
	if (!other.items & new)
		Deathmatch_Weapon (other, new);
// 1998-08-15 Improved weapon switching on pickups by Maddes  end
*/
	other.items = other.items | new;
	
	bound_other_ammo ();

	if (self.ammo_shells1)
	{
		if (acount)
			sprint(other, " and ");
		acount = 1;
		s = ftos(self.ammo_shells1);
		sprint (other, s);
		sprint (other, " shells");
	}
	if (self.ammo_nails1)
	{
		if (acount)
			sprint(other, " and ");
		acount = 1;
		s = ftos(self.ammo_nails1);
		sprint (other, s);
		sprint (other, " bullets");
	}
	if (self.ammo_rockets1)
	{
		if (acount)
			sprint(other, " and ");
		acount = 1;
		s = ftos(self.ammo_rockets1);
		sprint (other, s);
		sprint (other, " missiles");
	}
	if (self.ammo_cells1)
	{
		if (acount)
			sprint(other, " and ");
		acount = 1;
		s = ftos(self.ammo_cells1);
		sprint (other, s);
		sprint (other, " remote mines");
	}
	if (self.ammo_50cal)
	{
		if (acount)
			sprint(other, " and ");
		acount = 1;
		s = ftos(self.ammo_50cal);
		sprint (other, s);
		sprint (other, " 50.cal rounds");
	}

	sprint (other, "\n");
// backpack touch sound
	sound (other, CHAN_ITEM, "weapons/pkup.wav", 1, ATTN_NORM);
//	stuffcmd (other, "bf\n");

// remove the backpack, change self to the player
	remove(self);
	self = other;

// change to the weapon
/*
    if (self.button0)
        W_SetCurrentAmmo();
    else if (self.weapon == IT_SHOTGUN)
	    W_SetCurrentAmmo();
    else if (self.items & IT_NAILGUN)
	    W_SetCurrentAmmo();
    else if (self.items & IT_UZI)
	    W_SetCurrentAmmo();
    else if (self.items & IT_SUPER_NAILGUN)
	    W_SetCurrentAmmo();
    else if (self.items & IT_SUPER_SHOTGUN)
	    W_SetCurrentAmmo();
    else if (self.items & IT_GRENADE_LAUNCHER)
	    W_SetCurrentAmmo();
    else if (self.items & IT_ROCKET_LAUNCHER)
	    W_SetCurrentAmmo();
    else if (self.items & IT_LIGHTNING)
	    W_SetCurrentAmmo();
    else
        {*/
//            self.weapon = W_BestWeapon();
//            W_WeaponChange();
	UpdateAmmoCounts (self);

		if (self.zoomtoggle == 1)
			W_SetAmmoOnly();
		else
            W_SetCurrentAmmo();
//        }
/*
	if (!deathmatch)
        if (self.button0)
	       W_SetCurrentAmmo();
        else if (self.weapon == IT_SHOTGUN)
	       W_SetCurrentAmmo();
        else if (self.items & IT_NAILGUN)
	       W_SetCurrentAmmo();
        else if (self.items & IT_SUPER_NAILGUN)
	       W_SetCurrentAmmo();
        else if (self.items & IT_SUPER_SHOTGUN)
	       W_SetCurrentAmmo();
        else if (self.items & IT_GRENADE_LAUNCHER)
	       W_SetCurrentAmmo();
        else if (self.items & IT_ROCKET_LAUNCHER)
	       W_SetCurrentAmmo();
        else if (self.items & IT_LIGHTNING)
	       W_SetCurrentAmmo();
        else
        {
           self.weapon = new;
	       W_WeaponChange();
	       W_SetCurrentAmmo();
        }
	else
        if (self.button0)
	       W_SetCurrentAmmo();
        else if (self.weapon == IT_SHOTGUN)
	       W_SetCurrentAmmo();
        else if (self.items & IT_NAILGUN)
	       W_SetCurrentAmmo();
        else if (self.items & IT_SUPER_NAILGUN)
	       W_SetCurrentAmmo();
        else if (self.items & IT_SUPER_SHOTGUN)
	       W_SetCurrentAmmo();
        else if (self.items & IT_GRENADE_LAUNCHER)
	       W_SetCurrentAmmo();
        else if (self.items & IT_ROCKET_LAUNCHER)
	       W_SetCurrentAmmo();
        else if (self.items & IT_LIGHTNING)
	       W_SetCurrentAmmo();
        else
        {
           Deathmatch_Weapon (old, new);
           W_WeaponChange();
           W_SetCurrentAmmo();
        }
*/
};

/*
===============
WeaponDrop
===============
*/
void() WeaponDrop =
{
	local entity	item;

	// this is to compensate for the new way we're hacking the ammo
	// counts. If they're monsters, just copy shells->shells1 etc.
	if (self.flags & FL_MONSTER)
	{
		self.ammo_shells1 = self.ammo_shells;
		self.ammo_nails1 = self.ammo_nails;
		self.ammo_rockets1 = self.ammo_rockets;
		self.ammo_cells1 = self.ammo_cells;
	}

	if (!(self.ammo_shells1 + self.ammo_nails1 + self.ammo_rockets1 + self.ammo_cells1 + self.ammo_50cal))
		return;	// nothing in it

	item = spawn();
	item.origin = self.origin - '0 0 24';

	item.items = self.weapon;
	if (item.items == IT_AXE)
		item.netname = "Knife";
	else if (item.items == IT_TEKBOW)
		item.netname = "Tek Bow";
	else if (item.items == IT_SHOTGUN)
	{
		item.netname = "Pistol";
		setmodel (item, "progs/g_pistol.mdl");
    }
	else if (item.items == IT_SUPER_SHOTGUN)
	{
		item.netname = "Shotgun";
		setmodel (item, "progs/g_shot.mdl");
    }
	else if (item.items == IT_NAILGUN)
	{
		item.netname = "Assualt Rifle";
		setmodel (item, "progs/g_nail.mdl");
    }
	else if (item.items == IT_UZI)
	{
		item.netname = "Uzi";
		setmodel (item, "progs/g_uzi.mdl");
    }
	else if (item.items == IT_M99)
	{
		item.netname = "Sniper Rifle";
		setmodel (item, "progs/g_sniper.mdl");
    }
	else if (item.items == IT_SUPER_NAILGUN)
	{
		item.netname = "Mini Gun";
		setmodel (item, "progs/g_nail2.mdl");
    }
	else if (item.items == IT_GRENADE_LAUNCHER)
	{
		item.netname = "Grenade Launcher";
		setmodel (item, "progs/g_rock.mdl");
    }
	else if (item.items == IT_ROCKET_LAUNCHER)
	{
		item.netname = "Rocket Launcher";
		setmodel (item, "progs/g_rock2.mdl");
    }
	else if (item.items == IT_LIGHTNING)
	{
		item.netname = "Remote Mines";
		setmodel (item, "progs/g_light.mdl");
    }
	else
	{
		item.netname = "";
		setmodel (item, "progs/backpack.mdl");
    }

	item.ammo_shells1 = self.ammo_shells1;
	item.ammo_nails1 = self.ammo_nails1;
	item.ammo_rockets1 = self.ammo_rockets1;
	item.ammo_cells1 = self.ammo_cells1;

	item.ammo_50cal = self.ammo_50cal;

	item.velocity_z = 300;
	item.velocity_x = -100 + (random() * 200);
	item.velocity_y = -100 + (random() * 200);
	
	item.flags = FL_ITEM;
	item.solid = SOLID_TRIGGER;
	item.movetype = MOVETYPE_TOSS;
	setsize (item, '-16 -16 0', '16 16 56');
	item.touch = WeaponDropTouch;
	
	item.nextthink = time + 120;	// remove after 2 minutes
	item.think = SUB_Remove;
};
