/*
==============================================================================

Tiger HAP Eurocopper

==============================================================================
*/

$cd id1/models/tiger
$scale	1
$origin 0 0 24
$base base
$skin base

$frame Alive1 Alive2 Alive3 Alive4 Alive5
$frame Death1

/*
==============================================================================
If the player moves behind cover before the missile is launched, launch it
at the last visible spot with no velocity leading, in hopes that the player
will duck back out and catch it.
==============================================================================
*/

/*
=============
LaunchMissile

Sets the given entities velocity and angles so that it will hit self.enemy
if self.enemy maintains it's current velocity
0.1 is moderately accurate, 0.0 is totally accurate
=============
*/
void(entity missile, float mspeed, float accuracy) LaunchMissile =
{
	local	vector	vec, move;
	local	float	fly;

	makevectors (self.angles);

// set missile speed
	vec = self.enemy.origin + self.enemy.mins + self.enemy.size * 0.7 - missile.origin;

// calc aproximate time for missile to reach vec
	fly = vlen (vec) / mspeed;

// get the entities xy velocity
	move = self.enemy.velocity;
	move_z = 0;

// project the target forward in time
	vec = vec + move * fly;

	vec = normalize(vec);
	vec = vec + accuracy*v_up*(random()- 0.5) + accuracy*v_right*(random()- 0.5);

	missile.velocity = vec * mspeed;

	missile.angles = '0 0 0';
	missile.angles_y = vectoyaw(missile.velocity);

// set missile duration
	missile.nextthink = time + 5;
	missile.think = SUB_Remove;
};


void() Tiger_run1;
void() Tiger_side1;
void() TigerBulletEffect;

/*
=================
TigerCheckAttack
=================
*/
float()	TigerCheckAttack =
{
	local vector	spot1, spot2;
	local entity	targ;
	local float		chance;

	if (time < self.attack_finished)
		return FALSE;
	if (!enemy_vis)
		return FALSE;

	if (enemy_range == RANGE_FAR)
	{
		if (self.attack_state != AS_STRAIGHT)
		{
			self.attack_state = AS_STRAIGHT;
			Tiger_run1 ();
		}
		return FALSE;
	}

	targ = self.enemy;

// see if any entities are in the way of the shot
	spot1 = self.origin + self.view_ofs;
	spot2 = targ.origin + targ.view_ofs;

	traceline (spot1, spot2, FALSE, self);

	if (trace_ent != targ)
	{	// don't have a clear shot, so move to a side
		if (self.attack_state != AS_STRAIGHT)
		{
			self.attack_state = AS_STRAIGHT;
			Tiger_run1 ();
		}
		return FALSE;
	}

	if (enemy_range == RANGE_MELEE)
		chance = 0.9;
	else if (enemy_range == RANGE_NEAR)
		chance = 0.6;
	else if (enemy_range == RANGE_MID)
		chance = 0.2;
	else
		chance = 0;

	if (random () < chance)
	{
		self.attack_state = AS_MISSILE;
		return TRUE;
	}

	if (enemy_range == RANGE_MID)
	{
		if (self.attack_state != AS_STRAIGHT)
		{
			self.attack_state = AS_STRAIGHT;
			Tiger_run1 ();
		}
	}
	else
	{
		if (self.attack_state != AS_SLIDING)
		{
			self.attack_state = AS_SLIDING;
			Tiger_side1 ();
		}
	}

	return FALSE;
};

/*
=================
TigerAttackFinished
=================
*/
void()	TigerAttackFinished =
{
	if (enemy_range >= RANGE_MID || !enemy_vis)
	{
		self.attack_state = AS_STRAIGHT;
		self.think = Tiger_run1;
	}
	else
	{
		self.attack_state = AS_SLIDING;
		self.think = Tiger_side1;
	}
};

/*
==============================================================================

FAST ATTACKS

==============================================================================
*/

void() Tiger_FastFire =
{
	local vector		vec;
	local vector		dst;

	if (self.owner.health > 0)
	{
		self.owner.effects = self.owner.effects | EF_MUZZLEFLASH;

		makevectors (self.enemy.angles);
		dst = self.enemy.origin - 13*self.movedir;

		vec = normalize(dst - self.origin);
		sound (self, CHAN_WEAPON, "weapons/sgun1.wav", 1, ATTN_NORM);
		launch_spike (self.origin, vec);
		newmis.velocity = vec*600;
		newmis.owner = self.owner;
		newmis.classname = "wizspike";
		newmis.touch = T_MissileTouch;
		setmodel (newmis, "progs/missile.mdl");
		setsize (newmis, VEC_ORIGIN, VEC_ORIGIN);
	}

	remove (self);
};


void() Tiger_StartFast =
{
	local	entity	missile;

	sound (self, CHAN_WEAPON, "tiger/fire.wav", 1, ATTN_NORM);
	self.v_angle = self.angles;
	makevectors (self.angles);

	missile = spawn ();
	missile.owner = self;
	missile.nextthink = time + 0.6;
	setsize (missile, '0 0 0', '0 0 0');
	setorigin (missile, self.origin + '0 0 30' + v_forward*14 + v_right*56);
	missile.enemy = self.enemy;
	missile.nextthink = time + 0.8;
	missile.think = Tiger_FastFire;
	missile.movedir = v_right;

	missile = spawn ();
	missile.owner = self;
	missile.nextthink = time + 1;
	setsize (missile, '0 0 0', '0 0 0');
	setorigin (missile, self.origin + '0 0 30' + v_forward*14 + v_right* -56);
	missile.enemy = self.enemy;
	missile.nextthink = time + 0.3;
	missile.think = Tiger_FastFire;
	missile.movedir = VEC_ORIGIN - v_right;
};

void() Tiger_bullet_touch;

void() Tiger_bullet_touch =
{
    local float		rs;
    local float     d;

	if (other == self.owner)
		return;

	if (other.classname == "monster_army")
	{
		remove(self);
		return;
	}

	if (other.solid == SOLID_TRIGGER)
		return;	// trigger field, do nothing

	if (pointcontents(self.origin) == CONTENT_SKY)
	{
		remove(self);
		return;
	}

	// hit something that bleeds
	if (other.takedamage)
	{
	    if(other.solid == SOLID_BSP)
	    {
//		    spawn_touchblood (25, trace_ent);
		    remove(self);
        }
	    else
	        BloodSplat();

        if(skill == 3)
            d = 25;
        else if(skill == 2)
	        d = 25;
        else if(skill == 1)
	        d = 10;
        else
	        d = 5;

		T_Damage (other, self, self.owner, d);
	}
	else
	{
	    WriteByte (MSG_BROADCAST, SVC_TEMPENTITY);
		WriteByte (MSG_BROADCAST, TE_GUNSHOT);
	    WriteCoord (MSG_BROADCAST, self.origin_x);
	    WriteCoord (MSG_BROADCAST, self.origin_y);
	    WriteCoord (MSG_BROADCAST, self.origin_z);
	    Spark();

    	rs = rint ((random() * 8) + 1);
    	if (rs == 1)
    		self.noise = "weapons/ric1.wav";
    	if (rs == 2)
    		self.noise = "weapons/ric2.wav";
    	if (rs == 3)
    		self.noise = "weapons/ric3.wav";
    	if (rs == 4)
    		self.noise = "weapons/ric4.wav";
    	if (rs == 5)
    		self.noise = "weapons/ric5.wav";
    	if (rs == 6)
    		self.noise = "weapons/ric6.wav";
    	if (rs == 7)
    		self.noise = "weapons/ric7.wav";
    	if (rs == 8)
    		self.noise = "weapons/ric8.wav";
    	if (rs == 9)
    		self.noise = "weapons/ric9.wav";

    	sound (self, CHAN_AUTO, self.noise, 1, ATTN_NORM);
    }
};

void() TigerBulletEffect =
{
	local	entity bullet;

	bullet = spawn ();
	bullet.owner = self;
	bullet.movetype = MOVETYPE_FLYMISSILE;
	bullet.solid = SOLID_BBOX;
	bullet.classname = "bullet";

// set bullet speed

	makevectors (self.angles);

	bullet.velocity = normalize(self.enemy.origin - self.origin);
	if (skill > 2)
	    bullet.velocity = ((bullet.velocity * 2500 + ((v_right * 100) * crandom ())) + ((v_up * 50) * crandom ()));
	else if (skill == 2)
	    bullet.velocity = ((bullet.velocity * 2500 + ((v_right * 200) * crandom ())) + ((v_up * 100) * crandom ()));
	else
	    bullet.velocity = ((bullet.velocity * 2500 + ((v_right * 300) * crandom ())) + ((v_up * 200) * crandom ()));
	bullet.angles = vectoangles(bullet.velocity);

// set bullet fly function

	bullet.nextthink = time + 0.08;
	bullet.think = bullet_reposition;

	setmodel (bullet, "progs/bullet.mdl");
	bullet.touch = Tiger_bullet_touch;

	setsize (bullet, '0 0 0', '0 0 0');
	setorigin (bullet,(self.origin + ('0 0 22')));
};

void() Tiger_fire =
{
	local	vector	dir;
	local	entity	en;

	ai_face();

	sound (self, CHAN_WEAPON, "weapons/defire.wav", 1, ATTN_NORM);

	en = self.enemy;

	dir = en.origin - self.origin;
	dir = normalize (dir);

	self.v_angle = vectoangles (dir);

	if (self.v_angle_x > 180)
		self.v_angle_x = self.v_angle_x - 360; // corrects an ID mistake

	self.v_angle_x = self.v_angle_x * -1;     // corrects an ID mistake

	if (self.v_angle_x > 80)
		self.v_angle_x = 80;

	if (self.v_angle_x < -80)
		self.v_angle_x = -80;

    TigerBulletEffect();
//    eject_case ((((self.origin + (v_forward * 20))) + ('0 0 20')) + (v_right * 10),dir);
};
/*
void() Tiger_idlesound =
{
	local float wr;
	wr = random() * 5;

	if (self.waitmin < time)
	{
	 	self.waitmin = time + 2;
	 	if (wr > 4.5)
	 		sound (self, CHAN_VOICE, "tiger/idle.wav", 1,  ATTN_IDLE);
	 	if (wr < 1.5)
	 		sound (self, CHAN_VOICE, "tiger/idle.wav", 1, ATTN_IDLE);
	}
	return;
};
*/
void()	Tiger_hover1	=[	$Alive1,	Tiger_hover2	] {ai_stand();sound (self, CHAN_VOICE, "tiger/idle.wav", 1,  ATTN_IDLE);};
void()	Tiger_hover2	=[	$Alive2,	Tiger_hover3	] {ai_stand();};
void()	Tiger_hover3	=[	$Alive3,	Tiger_hover4	] {ai_stand();};
void()	Tiger_hover4	=[	$Alive4,	Tiger_hover5	] {ai_stand();};
void()	Tiger_hover5	=[	$Alive5,	Tiger_hover1	] {ai_stand();};

void()	Tiger_walk1		=[	$Alive1,	Tiger_walk2		] {ai_walk(16);sound (self, CHAN_VOICE, "tiger/idle.wav", 1,  ATTN_IDLE);};
void()	Tiger_walk2		=[	$Alive2,	Tiger_walk3		] {ai_walk(16);};
void()	Tiger_walk3		=[	$Alive3,	Tiger_walk4		] {ai_walk(16);};
void()	Tiger_walk4		=[	$Alive4,	Tiger_walk5		] {ai_walk(16);};
void()	Tiger_walk5		=[	$Alive5,	Tiger_walk1		] {ai_walk(16);};

void()	Tiger_side1		=[	$Alive1,	Tiger_side2		] {ai_run(32);sound (self, CHAN_VOICE, "tiger/idle.wav", 1,  ATTN_IDLE);};
void()	Tiger_side2		=[	$Alive2,	Tiger_side3		] {ai_run(32);};
void()	Tiger_side3		=[	$Alive3,	Tiger_side4		] {ai_run(32);};
void()	Tiger_side4		=[	$Alive4,	Tiger_side5		] {ai_run(32);};
void()	Tiger_side5		=[	$Alive5,	Tiger_side1		] {ai_run(32);};

void()	Tiger_run1		=[	$Alive1,	Tiger_run2		] {ai_run(8);sound (self, CHAN_VOICE, "tiger/idle.wav", 1,  ATTN_IDLE);};
void()	Tiger_run2		=[	$Alive2,	Tiger_run3		] {ai_run(8);};
void()	Tiger_run3		=[	$Alive3,	Tiger_run4		] {ai_run(8);};
void()	Tiger_run4		=[	$Alive4,	Tiger_run5		] {ai_run(8);};
void()	Tiger_run5		=[	$Alive5,	Tiger_run1		] {ai_run(8);};

void()	Tiger_fast1		=[	$Alive1,	Tiger_fast2		] {ai_face();Tiger_StartFast();sound (self, CHAN_VOICE, "tiger/idle.wav", 1,  ATTN_IDLE);};
void()	Tiger_fast2		=[	$Alive2,	Tiger_fast3		] {ai_face();};
void()	Tiger_fast3		=[	$Alive3,	Tiger_fast4		] {ai_face();};
void()	Tiger_fast4		=[	$Alive4,	Tiger_fast5		] {ai_face();};
void()	Tiger_fast5		=[	$Alive5,	Tiger_fast6		] {ai_face();};
void()	Tiger_fast6		=[	$Alive1,	Tiger_fast7		] {ai_face();};
void()	Tiger_fast7		=[	$Alive2,	Tiger_fast8		] {ai_face();};
void()	Tiger_fast8		=[	$Alive3,	Tiger_fast9		] {ai_face();};
void()	Tiger_fast9		=[	$Alive4,	Tiger_fast10	] {ai_face();};
void()	Tiger_fast10	=[	$Alive5,	Tiger_run1		] {ai_face();SUB_AttackFinished(2);TigerAttackFinished ();};

void()	Tiger_shoot1	=[	$Alive1,	Tiger_shoot2	] {ai_face();Tiger_fire();sound (self, CHAN_VOICE, "tiger/idle.wav", 1,  ATTN_IDLE);};
void()	Tiger_shoot2	=[	$Alive2,	Tiger_shoot3	] {ai_face();Tiger_fire();};
void()	Tiger_shoot3	=[	$Alive3,	Tiger_shoot4	] {ai_face();Tiger_fire();};
void()	Tiger_shoot4	=[	$Alive4,	Tiger_shoot5	] {ai_face();Tiger_fire();};
void()	Tiger_shoot5	=[	$Alive5,	Tiger_shoot6	] {ai_face();Tiger_fire();};
void()	Tiger_shoot6	=[	$Alive1,	Tiger_shoot7	] {ai_face();Tiger_fire();};
void()	Tiger_shoot7	=[	$Alive2,	Tiger_shoot8	] {ai_face();Tiger_fire();};
void()	Tiger_shoot8	=[	$Alive3,	Tiger_shoot9	] {ai_face();Tiger_fire();};
void()	Tiger_shoot9	=[	$Alive4,	Tiger_shoot10	] {ai_face();Tiger_fire();};
void()	Tiger_shoot10	=[	$Alive5,	Tiger_run1		] {ai_face();Tiger_fire();};

void()	Tiger_pain1		=[	$Alive1,	Tiger_pain2		] {sound (self, CHAN_VOICE, "tiger/idle.wav", 1,  ATTN_IDLE);};
void()	Tiger_pain2		=[	$Alive2,	Tiger_pain3		] {};
void()	Tiger_pain3		=[	$Alive3,	Tiger_pain4		] {};
void()	Tiger_pain4		=[	$Alive4,	Tiger_pain5		] {};
void()	Tiger_pain5		=[	$Alive5,	Tiger_run1		] {};

void()	Tiger_death1	=[	$Death1,	Tiger_death2	]
{

	self.velocity_x = -200 + 400*random();
	self.velocity_y = -200 + 400*random();
	self.velocity_z = 100 + 100*random();
	self.flags = self.flags - (self.flags & FL_ONGROUND);
//	sound (self, CHAN_VOICE, "wizard/wdeath.wav", 1, ATTN_NORM);

};
void()	Tiger_death2	=[	$Death1,	Tiger_death3	] {};
void()	Tiger_death3	=[	$Death1,	Tiger_death3	]
{
	setsize (self, '-64 -64 8', '64 64 64');

	if (pointcontents(self.origin) == CONTENT_SKY)
	{
		remove(self);
		return;
	}
	else if (pointcontents(self.origin) == CONTENT_SOLID)
	{
		setsize (self, '-64 -64 0', '64 64 64');
	}

/*self.solid = SOLID_NOT;*/

};

void() 	Tiger_die =
{
	local entity    explode;

	explode = spawn();

	explode.owner = self;
	explode.classname = "spike";

	explode.movetype = MOVETYPE_NOCLIP;
	explode.origin = self.origin + crandom()*v_forward*10 + v_up * 10 + crandom()*v_right*10;
	explode.velocity = '0 0 1';
	explode.velocity = self.velocity + crandom()*v_forward*10 + v_up * 50 + crandom()*v_right*10;
	explode.touch = SUB_Null;
	setmodel (explode, "progs/s_explod.spr");
	explode.solid = SOLID_NOT;

	explode.think =	s_explode1;
	explode.nextthink = time + 0.05;

	sound (self, CHAN_VOICE, "weapons/r_exp3.wav", 1, ATTN_NORM);

	T_RadiusDamage (self, self.owner, 100, world);

	WriteByte (MSG_BROADCAST, SVC_TEMPENTITY);
	WriteByte (MSG_BROADCAST, TE_EXPLOSION);
	WriteCoord (MSG_BROADCAST, self.origin_x);
	WriteCoord (MSG_BROADCAST, self.origin_y);
	WriteCoord (MSG_BROADCAST, self.origin_z);
	WriteCoord (MSG_BROADCAST, 1.0); // Red explosion colour
	WriteCoord (MSG_BROADCAST, 0.5); // Green explosion colour
	WriteCoord (MSG_BROADCAST, 0.0); // Blue explosion colour

	Tiger_death1 ();
};

void(entity attacker, float damage) Tiger_Pain =
{
//	sound (self, CHAN_VOICE, "wizard/wpain.wav", 1, ATTN_NORM);
	if (random()*70 > damage)
		return;		// didn't flinch

	Tiger_pain1 ();
};

void() Tiger_Missile =
{
	if (vlen (self.enemy.origin - self.origin) > 400)
	{
		if (random () < 0.5)
			Tiger_shoot1 ();
		else
			Tiger_fast1();
	}
	else
	{
		if (random () < 0.25)
			Tiger_shoot1 ();
		else
			Tiger_fast1();
	}
};

/*QUAKED monster_tiger (1 0 0) (-16 -16 -24) (16 16 40) Ambush
*/
void() monster_tiger
{
	if (deathmatch & DM_DM_MASK)
	{
		remove(self);
		return;
	}

	precache_model ("progs/tiger.mdl");

	precache_sound ("wizard/hit.wav");		// used by c code
	precache_sound ("tiger/fire.wav");
	precache_sound ("tiger/idle.wav");

	self.solid = SOLID_SLIDEBOX;
	self.movetype = MOVETYPE_STEP;

	setmodel (self, "progs/tiger.mdl");
	setsize (self, '-64 -64 0', '64 64 64');

	if (skill <= 1)
		self.health = 2500;
	else if (skill == 2)
		self.health = 5000;
	else
		self.health = 10000;

	self.th_stand = Tiger_hover1;
	self.th_walk = Tiger_walk1;
	self.th_run = Tiger_run1;
	self.th_missile = Tiger_Missile;
	self.th_pain = Tiger_Pain;
	self.th_die = Tiger_die;

	flymonster_start ();
};
